#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import time
import datetime

# ###########################################################
# Protokolliert die pro Minute durchschnittlich gesendete und
# empfangene Anzahl von Bytes auf dem tun0-Interface.
#
# Das Script ist via crontab zu starten:
#   *  * * * *   $HOME/bin/tun0moni.py
#
# Es werden täglich neue Logdateien unter ~/logs angelegt.
# Dateiname gemäß Tagesdatum. Also z.B. 2014_05_26.log
#
# Innerhalb des Logs dann das Format:
#   hh:mm empfangsdatenrate sendedatenrate
# also etwa:
#   17:59 2400233 326018
#   18:00 2256029 306306
#   18:01 3100856 327917
#
# D.A. 26.05.2014

ETH_IF="tun0"


# ###########################################################
# Verwaltung einer einfachen KeyValue-Datei.
class ConfigFile():
  # ###########################################################
  # Lädt die Datei "filename" in das Dictionary "self.cfgDict".
  def __init__(self, filename):
    self.filename=filename
    self.fobj=self.__open("r")
    if self.fobj!=None:
      lns=self.fobj.readlines()
      self.cfgDict=self.__stringListToDict(lns)
      self.fobj.close()
    else:
      self.cfgDict={}

  # ###########################################################
  # Liefert den Wert des Attributes "attr". Wenn das Attribut
  # nicht existiert, wird "noAttrVal" geliefert.
  def getAttr(self, attr, noAttrVal=None):
    return(self.cfgDict.get(attr, noAttrVal))

  # ###########################################################
  # Setzt das Attribut "attr" auf den Wert "val". Sofern
  # "writeToDisk" nicht auf False gesetzt wird, wird die
  # Datei danach neu auf Platte geschrieben.
  def setAttr(self, attr, val, writeToDisk=True):
    self.cfgDict.update({attr.upper() : val})
    if writeToDisk==True:
      self.fobj=self.__open("w")
      if self.fobj!=None:
        for a, v in self.cfgDict.items():
          self.fobj.write(a+"="+v+"\n")
        self.fobj.close()

  # ###########################################################
  # Wandelt eine Liste aus Strings im Format
  #    ["aa=bb", "cc=dd\n", ...]
  # in ein entspr. Dictionary im Format
  #    {"AA":"bb", "CC":"dd", ...}.
  def __stringListToDict(self, stringList):
    d={}
    for l in stringList:
      if l.find("=")>0:
        a, v=l.split("=", 1)
        d.update({a.upper() : v.rstrip()})
    return(d)

  # ###########################################################
  # Öffnet die Datei "self.filename" in dem Modus "mode".
  # Liefert das entspr. Dateiobjekt oder None, wenn die Datei
  # nicht geöffnet werden konnte.
  def __open(self, mode):
    try:
      fobj=open(self.filename, mode)
    except:
      fobj=None
    return(fobj)


# ###########################################################
# Traffic-Informationen von ETH_IF auslesen.
#
# pos=0     1           2         3    4    5    6     7          8           9        10      11   12   13   14    15      16
#interface: bytes       packets   errs drop fifo frame compressed multicast | bytes    packets errs drop fifo colls carrier compressed
#     eth0: 1339779969  998857    0    0    0    0     0          752         54799542 625934  0    0    0    0     1       0
#           rec_pos                                                           snd_pos
def read_proc_net_dev():
  fl=open("/proc/net/dev", "r")
  s=0
  r=0
  for ln in fl:             # über alle Zeilen aus /proc/net/dev (bzw. bis break)
    ln=ln.strip()           # führende/abschließende Leerzeichen killen
    if ln.find(ETH_IF)==0:  # wenn Zeile mit "tun0" beginnt...ist die interessante Zeile erreicht
      lnl=ln.split()        # String nach Liste von Worten wandeln
      r=long(lnl[1])        # empfangene Bytes und
      s=long(lnl[9])        # gesendete Bytes auslesen
      break
  fl.close()
  return((r, s))            # und als Tupel zurückliefern


# ###########################################################
# Die Haupt-Schleife.
if __name__=="__main__":
  homedir=os.path.expanduser('~')
  logfilename=os.path.join(homedir, "logs", time.strftime("%Y_%m_%d")+".log")
  lastvals=ConfigFile(os.path.join(homedir, ".tun0moni.conf"))

  tss=lastvals.getAttr("TIMESTAMP") # Zeitstempel des letzten Laufes
  ro=lastvals.getAttr("RECEIVED")   # alte Werte holen
  so=lastvals.getAttr("SENT")

  rn, sn=read_proc_net_dev()        # aktuelle Werte lesen

  if tss!=None: # wenn conf-file bereits existiert
    tsd=datetime.datetime.strptime(tss, "%Y.%m.%d-%H:%M:%S")  # Zeitstempel des letzten Laufes nach datetime 
    now=datetime.datetime.now()
    delta=(now-tsd).total_seconds()   # jetzt - letzter Lauf = Alter in Sekunden
    if 58<=delta<=62:                 # wenn es ca. eine Minute her ist
      r=max(int((rn-long(ro))/delta), 0)   # wenn tun0 "runter" ist, wird rn bzw. sn ==0 geliefert...
      s=max(int((sn-long(so))/delta), 0)   # somit würde r und s negativ werden - daher hier auf mindestens 0 setzen
      fobj=open(logfilename, "a")     # ...und protokollieren
      fobj.write("%s %d %d\n"%(time.strftime("%H:%M"), r, s))
      fobj.close()

  lastvals.setAttr("TIMESTAMP", time.strftime("%Y.%m.%d-%H:%M:%S"), writeToDisk=False)
  lastvals.setAttr("SENT", str(sn), writeToDisk=False)
  lastvals.setAttr("RECEIVED", str(rn))

